import os
from craftbuildtools import OperationPlugin


class CleanOperation(OperationPlugin):
    def __init__(self):
        super(CleanOperation, self).__init__()
        self.name = "clean_operation"
        self.description = "Rid yourself of those dirty old files and get some new ones"

    def perform(self, *args, **kwargs):
        from craftbuildtools import app, logger
        import click

        for project_jar in app.get_jar_files():
            os.remove(project_jar)
            click.echo("File %s has been removed" % os.path.basename(project_jar))


clean_operation_plugin = CleanOperation()



import os
from ftpretty import ftpretty
from craftbuildtools import OperationPlugin


class UploadOperation(OperationPlugin):
    def __init__(self):
        super(UploadOperation, self).__init__()
        self.name = "upload_operation"
        self.description = "Upload the files compiled or copied to a remote server!"

    def perform(self, *args, **kwargs):
        from craftbuildtools import app, logger
        import click

        files_to_copy = app.copied_files
        if len(files_to_copy) == 0:
            files_to_copy = app.get_jar_files()

        ftp_client = ftpretty(host=kwargs.pop("host"), user=kwargs.pop("user"),
                              password=kwargs.pop("password"))

        put_directory = kwargs.pop('directory')

        click.echo("Connected to FTP Remote Host; Uploading files to %s" % put_directory)

        ftp_client.cd(put_directory)

        for copied in files_to_copy:
            base_file = os.path.basename(copied)
            ftp_client.put(copied, base_file)
            click.echo("Uploaded %s to %s" % (base_file, put_directory))

        ftp_client.close()


upload_operation = UploadOperation()



import logging
import os
import click
import sys
import yaml
from simpleplugins import PluginManager, get_files_recursive

from craftbuildtools.operations import OperationPlugin
from craftbuildtools.data import Project

logger = logging.getLogger("CraftBuildTools")

formatter = logging.Formatter('%(asctime)s %(name)-12s %(levelname)-8s %(message)s')
console_log_handler = logging.StreamHandler(stream=sys.stdout)
console_log_handler.setFormatter(formatter)
console_log_handler.setLevel(logging.DEBUG)

logger.addHandler(console_log_handler)
logger.setLevel(logging.WARN)

import simpleplugins

# Assign the simple-plugins log-level to be of the same we have!
simpleplugins.logger.setLevel(logger.level)


class CraftBuildToolsApplication(object):
    def __init__(self):
        self.plugin_manager = PluginManager()
        self.config = None
        self.projects = {}
        self.build_projects = []
        self.built_projects = []
        self.copied_files = []
        self.app_config_folder = os.path.expanduser("~/.craftbuildtools/")
        self.files_folder = os.path.join(self.app_config_folder, "files")
        self.projects_folder = os.path.join(self.app_config_folder, "projects")

        if not os.path.exists(self.app_config_folder):
            os.makedirs(self.app_config_folder)
            logger.debug("Created app config folder at %s" % self.app_config_folder)

        if not os.path.exists(self.files_folder):
            os.makedirs(self.files_folder)

        if not os.path.exists(self.projects_folder):
            os.makedirs(self.projects_folder)
            logger.debug("Created projects folder at %s" % self.projects_folder)

        self.config_location = os.path.join(self.app_config_folder, "config.yml")

        self.__init_config()

        self.__init_files_folder()

        self.__load_plugins()

        self.__load_projects()

    def __load_plugins(self):
        self.plugin_manager.register(directory=os.path.abspath(os.path.join(os.path.dirname(__file__), 'operations')),
                                     skip_types=[OperationPlugin])

    def __init_config(self):
        if not os.path.isfile(self.config_location):
            self.__create_config()

        self.__load_config()

    def __init_files_folder(self):
        if not os.path.exists(self.files_folder):
            os.mkdir(self.files_folder)
            logger.debug("Created the Files folder at %s" % self.files_folder)

    def get_jar_files(self):
        return get_files_recursive(self.files_folder, "*.jar")

    def __load_projects(self):
        # For some reason matching against .yml files doesn't work, so let's look for all files.
        project_config_files = get_files_recursive(self.projects_folder, match='*')

        # There's no projects files to load, so let's make a dummy file and save it.
        if len(project_config_files) == 0:

            # Call the add projects plugin to perform the operation for a
            self.plugin_manager.get_plugin("addproject_operation").perform()

            for project in self.projects:
                with open(os.path.join(self.projects_folder, '%s.yml' % project.name), 'w') as project_new_config_file:
                    yaml.dump(project.yaml(), project_new_config_file, default_flow_style=False)

                logger.debug("Created Project File %s.yml" % project.name)

        # Now we loop through all the files in the projects directory and load them!
        for project_config_file in project_config_files:
            project = None
            with open(project_config_file, 'r') as project_file:
                project = Project.load(project_file)

                if project.name in self.projects:
                    logger.warning("Duplicate Project found [%s]" % project.name)
                    continue
                self.projects[project.name] = project

        logger.debug("Loaded Projects [%s] from config folder." % ",".join(name for name in self.projects.keys()))

    def __create_config(self):
        if not self.config:
            self.config = {
                'ftp-host': click.prompt('FTP Host', default='HOST'),
                'ftp-username': click.prompt('FTP Username', default='USERNAME'),
                'ftp-password': click.prompt('FTP Password', default='PASSWORD'),
                'remote-upload-directory': click.prompt('Remote upload directory', default='/Dev/plugins/')
            }
            # TODO implement config options to load templates and their generators

        with open(self.config_location, 'w') as yaml_file:
            yaml.dump(self.config, yaml_file, default_flow_style=False)

        logger.debug("Configuration for CraftBuildTools has been saved at %s" % self.config_location)

    def __load_config(self):
        with open(self.config_location, 'r') as yaml_file:
            self.config = yaml.load(yaml_file)

    def save_project(self, project):
        with open(os.path.join(self.app_config_folder, '%s.yml' % project.name), 'w') as project_new_config_file:
            yaml.dump(project.yaml(), project_new_config_file, default_flow_style=False)


app = CraftBuildToolsApplication()


@click.group(chain=True, add_help_option=True)
def cli():
    """
    \b
    About CraftBuildTools
    ----------------
    A wonderful suite of commands and utilities used to aid
    your development with Minecraft Projects.


    \b
    Features
    --------
        * Project Manager (To cache your project Information)
        * Clean your old project builds, and prepare for the new!
        * Build your projects! Not one at a time, but multiple!
        * Copy your built projects to a specific location!
        * Upload your built projects to a remote FTP Server!
    \b
        * Create Projects from Templates
            Reduce the amount of time it takes for
            you to get coding by giving CraftBuildTools a template
            to create your new project by!
            It will handle the entire process for you.
\b
        * Plugin-Based design!
            Rip out and replace any parts of the suite
            you wish; Changing their functionality but still allowing
            the rest of the app to function as it should!
\b
        * Lightweight!
            This is a big key! Having a lightweight
            CLI tool allows you to have maximum productivity
            leaving the pre-project boilerplate, and post-build
            actions entirely up to this tool!
    """


@cli.command("projects", help="""
\b
    Perform operations on a project.
    Select one of the following options:
      - list
      - add
      - edit
      - remove
    """)
@click.argument("action", type=click.STRING, default=None, required=False)
def projects_command(action):
    plugins_by_action = {
        'list': app.plugin_manager.get_plugin('listprojects_operation'),
        'add': app.plugin_manager.get_plugin('addproject_operation'),
        'edit': app.plugin_manager.get_plugin('editproject_operation'),
        'remove': app.plugin_manager.get_plugin('removeproject_operation')
    }

    if action is None:
        while True:
            action = click.prompt("Choose an action to perform (list, add, edit, remove)")
            if action in plugins_by_action:
                break
            else:
                click.clear()
                click.echo("%s is not a valid action; Try again!" % action)

    plugin = plugins_by_action[action]
    plugin.perform()


@cli.command(help="Clean all those old project files, and builds! (Think: mvn clean)")
def clean():
    app.plugin_manager.get_plugin("clean_operation").perform()


@cli.command(help="Build your projects in succession")
@click.option('-p', '--project', "projects", metavar="<name>", multiple=True, help="Project(s) to build")
def build(projects):
    app.build_projects.clear()

    if "all" in projects:
        for name in app.projects.keys():
            app.build_projects.append(name)
    else:
        for build_name in projects:
            if build_name not in app.projects:
                logger.debug("No project with named %s exists" % build_name)
                continue

            app.build_projects.append(build_name)
            logger.debug("Projects '%s' is to be built!" % build_name)

    app.plugin_manager.get_plugin("build_operation").perform()


@cli.command(help="To be used after build; Copy your built project files to a specific location")
@click.option("-l", "--location", "location", type=click.Path(), default=app.files_folder,
              help="Location to copy the built files to.")
def copy(location):
    app.files_folder = os.path.expanduser(location)

    if not os.path.exists(app.files_folder):
        os.makedirs(app.files_folder)

    app.plugin_manager.get_plugin("copy_operation").perform()


@cli.command("upload", help="""
Enabled you to upload your projects to a remote FTP Server after they've been built!

If the options are not specified, default values will be polled from the application config.
""")
@click.option("--updateconfig", "updateconfig", type=click.BOOL, default=False,
              help="Save the configuration values to file, for re-use")
@click.option('-h', "host", type=click.STRING, default=app.config['ftp-host'], help="FTP Host Name", metavar="<host>")
@click.option('-u', "user", type=click.STRING, default=app.config['ftp-username'], help="FTP username",
              metavar="<user>")
@click.option('-p', "password", type=click.STRING, default=app.config['ftp-password'], help="FTP Password",
              metavar="<password>")
@click.option('-d', "directory", type=click.STRING, default=app.config['remote-upload-directory'],
              help="Remote Folder for Files", metavar="<directory>")
def upload(host, user, password, directory, updateconfig):
    if updateconfig:
        app.config['ftp-host'] = host
        app.config['ftp-host'] = user
        app.config['ftp-password'] = password
        app.config['remote-upload-directory'] = directory
        logger.info("Updated FTP Configuration to the options provided.")

    click.echo("Beginning Upload Operation!")

    app.plugin_manager.get_plugin("upload_operation").perform(host=host, user=user, password=password,
                                                              directory=directory)


@cli.command(help="Generate a new project from a template! Get to coding quicker!")
@click.option("-tp", "--template", "template", metavar="<name>", type=click.STRING, required=False,
              help="Template to create the project with")
@click.option("-tf", "--templatefolder", "templatefolder", type=click.Path(),
              default=os.path.join(app.app_config_folder, "templates/"), help="Path where the template is stored")
@click.option("--clone", "git_clone_repo", metavar="<git-repo>", type=click.STRING, required=False,
              help="Clone down a git repo and use that as a template")
def createproject(template=None, templatefolder=None, git_clone_repo=None):
    app.plugin_manager.get_plugin("createproject_operation").perform(
        template=template,
        templatefolder=templatefolder,
        git_clone_repo=git_clone_repo
    )



import inspect
import os
import click
from craftbuildtools import OperationPlugin
from craftbuildtools.template import TemplateRenderPlugin


class CreateProjectOperation(OperationPlugin):
    def __init__(self):
        super(CreateProjectOperation, self).__init__()
        self.name = "createproject_operation"
        self.description = "Generate a project for you to hack on via a CookieCutter Template"

    def perform(self, *args, **kwargs):
        from craftbuildtools import app, logger
        from craftbuildtools.utils import ChangeDir, get_filename

        templates_folder = kwargs.pop("templatefolder")
        template = kwargs.pop("template")
        git_repo = kwargs.pop("git_clone_repo")

        if not os.path.exists(templates_folder):
            os.makedirs(templates_folder)

        new_git_repo = click.prompt("Would you like to add a new template?", default=False, type=click.BOOL)

        if new_git_repo:
            git_repo = click.prompt("Enter the URL For your git-repo to use as a template", type=click.STRING)

        # If there was a git repository included in the arguments, then
        # Attempt to clone this done!
        if git_repo is not None:
            # Enter the Templates Folder!
            with ChangeDir(templates_folder):
                # Get the name of the folder, from the git repo.
                cloned_folder_name = get_filename(git_repo)
                # If the git repository doesn't exist then we need to clone it doen.
                if not os.path.exists(os.path.join(templates_folder, cloned_folder_name)):
                    git.clone(git_repo)

                    if not os.path.exists(os.path.join(templates_folder, cloned_folder_name).replace(".git", "")):
                        logger.error("Unable to find cloned repo %s at expected location %s" % (
                            cloned_folder_name, os.path.join(templates_folder, cloned_folder_name)))
                        exit()
                        return
                else:
                    click.echo("Git Repo %s already exists at %s" % (cloned_folder_name, templates_folder))

        available_templates = []

        for file in os.listdir(templates_folder):
            file = os.path.join(templates_folder, file)
            logger.debug("First Loop File: %s" % file)
            if os.path.isdir(file):
                logger.debug("%s is dir" % file)
                available_templates.append(file)

        # If there's no template or git_repo available, and no projects
        # In the available templates folder.
        if len(available_templates) == 0:
            click.echo("A template is required in order to create a project.")
            wants_to_clone = click.prompt("Would you like to clone a git repo for your template?", default=True,
                                          type=click.BOOL)

            try:
                from sh import git
            except:
                click.echo(
                    "You must have git installed on your machine in order to perform a clone. Aborting. Install git and try again")

                exit()
                return

            if not wants_to_clone:
                clone_commons_templates = click.prompt("Would you like to clone the default repositories?",
                                                       default=True, type=click.BOOL)

                if not clone_commons_templates:
                    logger.error("You must have atleast one available template in-order to create a project")
                    exit()
                    return

                with ChangeDir(templates_folder):
                    # TODO Read list of default repositories from config on git. Or something.

                    cloned = 0

                    if not os.path.exists(os.path.join(templates_folder, "cookiecutter-commons-bukkitplugin")):
                        logger.info("Cloning CookieCutter Template: Commons (BukkitPlugin)")
                        git.clone('https://github.com/TechnicalBro/cookiecutter-commons-bukkitplugin.git')
                        cloned += 1

                    if not os.path.exists(os.path.join(templates_folder, "cookiecutter-commons-minigame")):
                        logger.info("Cloning CookieCutter Template: Commons (MiniGame)")
                        git.clone("https://github.com/TechnicalBro/cookiecutter-commons-minigame.git")
                        cloned += 1

                    if not os.path.exists(os.path.join(templates_folder, "cookiecutter-bukkit-maven")):
                        logger.info("Cloning CookieCutter Template: Bukkit Plugin")
                        git.clone("https://github.com/TechnicalBro/cookiecutter-bukkit-maven.git")
                        cloned += 1

                    if cloned > 0:
                        logger.info("Finished Cloning available repositories.")

            else:
                git_repo = click.prompt("Enter the URL of the git repo to clone", type=click.STRING)

                with ChangeDir(templates_folder):
                    # Get the name of the folder, from the git repo.
                    cloned_folder_name = get_filename(git_repo)
                    # If the git repository doesn't exist then we need to clone it doen.
                    if not os.path.exists(os.path.join(templates_folder, cloned_folder_name)):
                        git.clone(git_repo)

                        if not os.path.exists(os.path.join(templates_folder, cloned_folder_name).replace(".git", "")):
                            logger.error("Unable to find cloned repo %s at expected location %s" % (
                                cloned_folder_name, os.path.join(templates_folder, cloned_folder_name)))
                            exit()
                            return
                    else:
                        click.echo("Git Repo %s already exists at %s" % (cloned_folder_name, templates_folder))

        available_templates = []

        logger.debug("Templates Folder is %s" % templates_folder)

        for file in os.listdir(templates_folder):
            file = os.path.join(templates_folder, file)

            if os.path.isdir(file):
                available_templates.append(file)
                logger.debug("Available Template: %s" % file)
            else:
                logger.debug("%s is not a directory" % file)

        if len(available_templates) == 0:
            click.echo("There's no available templates to choose from. Exiting")
            exit()

        templates_data = {}

        app.plugin_manager.register(directory=templates_folder, skip_types=OperationPlugin, override=True)

        if not app.plugin_manager.has_plugin(plugin_type=TemplateRenderPlugin):
            logger.error(
                "Unable to locate template rendering plugins in Plugin Manager. Assure your template has a script to render it by.")
            exit()
            return

        template_plugins = app.plugin_manager.get_plugins(plugin_type=TemplateRenderPlugin)
        for plugin in template_plugins:
            templates_data[plugin.template_name] = {
                'plugin': plugin,
                'folder': os.path.dirname(inspect.getfile(plugin.__class__))
            }

        logger.debug("Template Data map is: %s" % str(templates_data))

        project_type = template
        if project_type not in template_plugins:
            project_type_prompt_text = "Which Template Would you like to Render? Choose from ({})".format(
                ', '.join(templates_data.keys()))

            project_type = click.prompt(project_type_prompt_text, type=click.Choice(templates_data.keys()))

        template_plugin = templates_data[project_type]['plugin']
        template_plugin.perform(directory=templates_data[project_type]['folder'])


create_project_operation_plugin = CreateProjectOperation()



import os
import subprocess
from craftbuildtools.operations import OperationPlugin
from craftbuildtools.utils import ChangeDir


class MavenBuildOperation(OperationPlugin):
    def __init__(self):
        super(MavenBuildOperation, self).__init__()
        self.name = "build_operation"
        self.description = "Mange your projects and generate builds (Maven Only)"

    def perform(self, *args, **kwargs):
        from craftbuildtools import app, logger
        import click

        failed_builds = []
        successful_builds = []
        invalid_project_folders = []

        app.build_projects.sort()

        total_project_count = 0

        from threading import Thread

        def call_maven_build(bp):
            with ChangeDir(bp.directory):
                click.echo("Executing build command '%s' on Project %s" % (bp.build_command, bp.name))

                # TODO Implement timer to see how long build starts, or spawn in a subprocess.
                build_process = subprocess.Popen(bp.build_command, shell=True, stdout=subprocess.PIPE,
                                                 stderr=subprocess.STDOUT)
                logger.debug("Build process has been spawned %s" % build_process.pid)

                build_success = False
                for line in build_process.stdout.readlines():
                    if b"BUILD SUCCESS" in line:
                        logger.debug("Maven Build Success in line: '%s'" % line)
                        build_success = True
                        break

                build_process.wait()

                if build_success is True:
                    click.echo("Project %s has been built successfully" % bp.name)
                    successful_builds.append(bp.name)
                    app.built_projects.append(bp)
                else:
                    click.echo("Project %s has failed to build" % bp.name)
                    failed_builds.append(bp.name)

        for project_name in app.build_projects:
            total_project_count += 1
            project = app.projects[project_name]

            if not os.path.exists(project.directory):
                invalid_project_folders.append(project.name)
                click.echo("Project %s folder (%s) doesn't exist... Is it valid?" % (project.name, project.directory))
                continue

            build_thread = Thread(target=call_maven_build, args=(project,))
            build_thread.start()
            logger.debug("Thread to build %s has been executed" % project.name)
            build_thread.join()
            logger.debug("Build Thread for %s has expired" % project.name)

        failed_projects = len(failed_builds)
        built_project = total_project_count - failed_projects - len(invalid_project_folders)
        click.echo(
            "BUILD OPERATION COMPLETE\nInvalid Projects: %s\nSuccessful Builds: %s\n\tNames: %s\nFailed Builds: %s\n\tNames: %s" %
            (",".join(name for name in invalid_project_folders),
             built_project,
             ",".join(name for name in successful_builds),
             failed_projects,
             ','.join(name for name in failed_builds)
             ))


build_plugin = MavenBuildOperation()



import os
from bs4 import BeautifulSoup
import click
import yaml
from craftbuildtools.utils import get_files_recursive
import craftbuildtools as cbt


class Project(object):
    def __init__(self, name, directory, target_directory, build_command="mvn clean install"):
        self.name = name
        self.directory = os.path.expanduser(directory)
        self.target_directory = os.path.expanduser(target_directory)
        self.build_command = build_command

    def yaml(self):
        return {
            'name': self.name,
            'directory': self.directory,
            'target_directory': self.target_directory,
            'build_command': self.build_command
        }

    @classmethod
    def create_from_prompt(cls):
        click.echo("Please enter your project information!")
        name = click.prompt("Project Name")
        directory = click.prompt("Project Directory")
        target_directory = click.prompt("Project Build Directory", default=os.path.join(directory, "target"))

        build_command = click.prompt("Project Build Command", default="mvn clean install")
        return Project(name=name, directory=directory, target_directory=target_directory, build_command=build_command)

    @staticmethod
    def load(data):
        values = yaml.safe_load(data)
        return Project(
            name=values['name'],
            directory=values['directory'],
            target_directory=values['target_directory'],
            build_command=values['build_command']
        )

    def __get_pom_file(self):
        pom_path = os.path.join(self.directory, "pom.xml")
        if not os.path.exists(pom_path):
            cbt.logger.warn("Project %s has no pom.xml at expects '%s'" % (self.name, pom_path))
            return None

        return pom_path

    def __has_pom_file(self):
        return self.__get_pom_file() is not None

    def get_pom_info(self):
        pom_file = self.__get_pom_file()
        if pom_file is None:
            cbt.logger.warn("Unable to find pom file for project %s" % self.name)
            return None

        pom_doc = None
        with open(pom_file, 'r') as pom_xml_file:
            pom_doc = pom_xml_file.read()

        if pom_doc is None:
            return None

        soup = BeautifulSoup(pom_doc, 'lxml')

        has_parent = len(soup.find_all("parent")) > 0

        artifact_id = None
        version = None
        if has_parent:
            art_count = 0
            for artifact_ids in soup.find_all("artifactid"):
                if art_count == 0:
                    art_count += 1
                    continue
                else:
                    artifact_id = artifact_ids.string
                    break

            ver_count = 0
            for versions in soup.find_all("version"):
                if ver_count == 0:
                    ver_count += 1
                    continue
                else:
                    version = versions.string
                    break

        else:
            artifact_id = soup.find("artifactid").string
            version = soup.find('version').string

        output_jar = "%s-%s.jar" % (artifact_id, version)

        # TODO parse more pom info.
        return {
            'output_jar': output_jar,
            'version': version,
            'artifact_id': artifact_id
        }

    def get_files(self, match="*.*"):
        return get_files_recursive(self.directory, match=match)

    def __str__(self):
        return """*-- %s --*
    * Directory: %s
    * Build Command %s
    * Version: %s
        """ % (
            self.name,
            self.directory,
            self.build_command,
            self.get_pom_info()['version']
        )

    def __repr__(self):
        return self.__str__()



from simpleplugins import Plugin


class OperationPlugin(Plugin):
    def __init__(self, **kwargs):
        super(OperationPlugin).__init__(**kwargs)
        self.command_delegate = None

    def perform(self, *args, **kwargs):
        raise NotImplementedError("Perform is not implemented in base class (OperationPlugin)")

    def activate(self):
        self.active = True

    def deactivate(self):
        self.active = False



import copy
import os
import click
from craftbuildtools.operations import OperationPlugin
from craftbuildtools.data import Project


class EditProjectOperation(OperationPlugin):
    def __init__(self):
        super(EditProjectOperation, self).__init__()
        self.name = "editproject_operation"
        self.description = "Edit a projects information"

    def perform(self, *args, **kwargs):
        from craftbuildtools import app, logger

        project_name = kwargs.pop("project_name", None)
        if project_name is None:
            available_projects = [name for name in app.projects.keys()]

            while True:
                click.echo("Available Projects: %s" % ",".join(p for p in available_projects))
                project_name = click.prompt("Which project would you like to Edit?", prompt_suffix=" ")
                if project_name in available_projects:
                    break
                else:
                    click.echo("%s is not a valid project!" % project_name)
                    click.clear()
                    click.echo("")

        project_info = app.projects[project_name]

        new_info = copy.deepcopy(project_info)

        new_info.name = click.prompt(text="Project Name", default=project_info.name, type=click.STRING)

        new_info.directory = click.prompt(text="Project Directory", default=project_info.directory,
                                          type=click.STRING)

        new_info.target_directory = click.prompt(text="Project Target Directory",
                                                 default=project_info.target_directory, type=click.STRING)

        new_info.build_command = click.prompt(text="Build Command", default=project_info.build_command,
                                              type=click.STRING)

        save_changes = click.prompt("Save Changes?", confirmation_prompt=True, type=click.BOOL)

        if not save_changes:
            click.echo("Changes to Project %s have been disregarded" % project_info.name)
            return

        del app.projects[project_name]
        import shutil
        import yaml

        config_path = os.path.join(app.projects_folder, "%s.yml" % project_info.name)

        app.save_project(new_info)
        click.echo("Project Information has been Updated!")

        if new_info.name != project_info.name:
            shutil.move(config_path, os.path.join(app.projects_folder, "%s.yml" % new_info.name))
            click.echo(
                "Previous config file %s has been renamed to %s" % (
                    "%s.yml" % project_info.name, "%s.yml" % new_info.name)
            )


add_project_operation_plugin = EditProjectOperation()



from craftbuildtools.operations import OperationPlugin


class ListProjectsPlugin(OperationPlugin):
    def __init__(self):
        super(ListProjectsPlugin, self).__init__()
        self.name = "listprojects_operation"
        self.version = "1",
        self.description = "List all the available projects to perform operations upon"

    def activate(self):
        pass

    def deactivate(self):
        pass

    def perform(self, *args, **kwargs):
        from craftbuildtools import app, logger
        import click

        for project_name in app.projects.keys():
            project = app.projects[project_name]
            click.echo(project)


list_projects_operation_plugin = ListProjectsPlugin()



import os
import click
from craftbuildtools.operations import OperationPlugin
from craftbuildtools.data import Project


class RemoveProjectOperation(OperationPlugin):
    def __init__(self):
        super(RemoveProjectOperation, self).__init__()
        self.name = "removeproject_operation"
        self.description = "Remove a project from the management of CraftBuildTools"

    def perform(self, *args, **kwargs):
        from craftbuildtools import app, logger

        project_name = kwargs.pop("project_name", None)
        if project_name is None:
            available_projects = [name for name in app.projects.keys()]

            while True:
                click.echo("Available Projects: %s" % ",".join(p for p in available_projects))
                project_name = click.prompt("Which project would you like to remove?", prompt_suffix=" ")
                if project_name in available_projects:
                    break
                else:
                    click.echo("%s is not a valid project!" % project_name)
                    click.clear()
                    click.echo("")

        project_info = app.projects[project_name]

        remove_project_file = click.prompt(text="Are you sure you want to remove the project '%s'?" % project_info.name,
                                           confirmation_prompt=True, type=click.BOOL, prompt_suffix=" ")

        if not remove_project_file:
            return

        import shutil

        os.remove(os.path.join(app.projects_folder, '%s.yml' % project_info.name))
        click.echo("Project '%s' file '%s.yml' has been removed." % (project_info.name, project_info.name))

        remove_project_source_files = click.prompt("Would you like to remove the project source files?",
                                                   type=click.BOOL, default=False)

        if remove_project_source_files:
            remove_projects = click.prompt(
                "Are you sure you want to remove the project files at (%s)?" % project_info.directory,
                type=click.BOOL, confirmation_prompt=True)

            if remove_projects:
                try:
                    shutil.rmtree(project_info.directory)
                    click.echo("Files at %s has been removed" % project_info.directory)
                except:
                    click.echo("Unable to locate project files. Nothing was removed.")

add_project_operation_plugin = RemoveProjectOperation()



import os
import yaml
from craftbuildtools.operations import OperationPlugin
from craftbuildtools.data import Project


class AddProjectOperation(OperationPlugin):
    def __init__(self):
        super(AddProjectOperation, self).__init__()
        self.name = "addproject_operation"
        self.description = "Add a project to be managed"

    def perform(self, *args, **kwargs):
        from craftbuildtools import app, logger

        projects_config_dir = os.path.join(app.app_config_folder, 'projects')
        project = Project.create_from_prompt()

        with open(os.path.join(projects_config_dir, "%s.yml" % project.name), "w") as project_new_config_file:
            yaml.dump(project.yaml(), project_new_config_file, default_flow_style=False)

        app.projects[project.name] = project
        logger.info("Project %s has been added to CraftBuildTools" % project.name)


add_project_operation_plugin = AddProjectOperation()



from simpleplugins import Plugin


class TemplateRenderPlugin(Plugin):
    def __init__(self, **kwargs):
        super(TemplateRenderPlugin, self).__init__(**kwargs)
        self.template_name = kwargs.pop("template_name")

    def deactivate(self):
        self.active = False

    def perform(self, **kwargs):
        raise NotImplementedError("Unable to render template; Method to handle rendering is not implemented.")

    def activate(self):
        self.active = True



import fnmatch
import os
from urllib.parse import urlsplit, urlparse


def save_python_script(script_folder, script_url):
    import requests

    if not os.path.exists(script_folder):
        os.makedirs(script_folder)

    script_name = get_filename(script_url)
    script_data = requests.get(script_url).text

    script_loc = os.path.join(script_folder, script_name)

    write_file(script_loc, script_data)

    if not os.path.exists(script_loc):
        raise FileNotFoundError("Unable to locate file %s after attempting to save it" % script_loc)

    return script_loc


def is_url(url):
    return urlparse(url).scheme != ""


def get_filename(url_or_path):
    if not is_url(url_or_path):
        if not os.path.exists(url_or_path):
            return None
        return "%s%s" % os.path.splitext(url_or_path)
    else:
        return "%s%s" % os.path.splitext(os.path.basename(urlsplit(url_or_path).path))


def get_file_extension(path):
    if is_url(path):
        return "%s" % os.path.splitext(os.path.basename(urlsplit(path).path))[1]
    else:
        if not os.path.exists(path):
            return None
        return "%s" % os.path.splitext(path)[1]


def get_files_recursive(path, match='*.py'):
    matches = []
    for root, dirnames, filenames in os.walk(path):
        for filename in fnmatch.filter(filenames, match):
            matches.append(os.path.join(root, filename))

    return matches


def get_config_from_file(file, trim_newlines=True):
    with open(file, 'r') as config_file:
        data = config_file.read()
        if trim_newlines:
            data = data.replace('\n', '')
        return data


def write_file(file, data):
    with open(file, 'w') as data_file:
        data_file.write(data)


class ChangeDir:
    def __init__(self, newPath):
        self.newPath = os.path.expanduser(newPath)

    # Change directory with the new path
    def __enter__(self):
        self.savedPath = os.getcwd()
        os.chdir(self.newPath)

    # Return back to previous directory
    def __exit__(self, etype, value, traceback):
        os.chdir(self.savedPath)


class Map(dict):
    """
    Example:
    m = Map({'first_name': 'Eduardo'}, last_name='Pool', age=24, sports=['Soccer'])
    """

    def __init__(self, *args, **kwargs):
        super(Map, self).__init__(*args, **kwargs)
        for arg in args:
            if isinstance(arg, dict):
                for k, v in arg.iteritems():
                    self[k] = v

        if kwargs:
            for k, v in kwargs.iteritems():
                self[k] = v

    def __getattr__(self, attr):
        return self.get(attr)

    def __setattr__(self, key, value):
        self.__setitem__(key, value)

    def __setitem__(self, key, value):
        super(Map, self).__setitem__(key, value)
        self.__dict__.update({key: value})

    def __delattr__(self, item):
        self.__delitem__(item)

    def __delitem__(self, key):
        super(Map, self).__delitem__(key)
        del self.__dict__[key]



#!/usr/bin/python3
from setuptools import setup


def main():
    setup(
        name='craftbuildtools',
        version='0.1.0',
        packages=[
            'craftbuildtools',
            'craftbuildtools.data',
            'craftbuildtools.operations',
            'craftbuildtools.template',
            'craftbuildtools.utils'
        ],
        url='https://github.com/TechnicalBro/CraftBuildTools',
        license='LICENSE.txt',
        author='Brandon Curtis',
        author_email='freebird.brandon@gmail.com',
        description='Build automation and Project creation for Minecraft/Spigot/Bukkit, Maven Projects',
        entry_points={
            'console_scripts': [
                'craftbuildtools = craftbuildtools.__main__:main'
            ]
        },
        keywords=[
            "minecraft", "spigot", "bukkit", "templates", "buildtools", "craftbuildtools"
        ],
        install_requires=[
            'beautifulsoup4==4.4.1',
            'argparse==1.4.0',
            'ftpretty==0.2.2',
            'pyyaml==3.11',
            'requests',
            'lxml==3.5.0',
            'cssselect==0.9.1',
            'cookiecutter==1.3.0',
            'click==6.2',
            'sh==1.11',
            'simpleplugins',
            'yamlbro',
        ],
        dependency_links=[
            'git+https://github.com/TechnicalBro/yaml-bro.git',
            'git+https://github.com/TechnicalBro/simple-python-plugins.git'
        ],
        test_suite='tests',
        tests_require=[
            'py',
            'pytest',
            'pytest-cov',
            'pytest-random'
        ],
    )


if __name__ == "__main__":
    main()



from craftbuildtools import OperationPlugin


class CopyOperation(OperationPlugin):
    def __init__(self):
        super(CopyOperation, self).__init__()
        self.name = "copy_operation"
        self.description = "Copy all your brand new, brilliant files to a new location!"

    def perform(self, *args, **kwargs):
        from craftbuildtools import app, logger
        import shutil
        import os
        import click
        from craftbuildtools.utils import get_filename

        copied_files = []

        for project in app.built_projects:
            pom_info = project.get_pom_info()
            output_jar_path = os.path.join(project.target_directory, pom_info['output_jar'])
            if not os.path.exists(output_jar_path):
                click.echo("Unable to find %s for project %s" % (pom_info['output_jar'], project.name))
                continue

            new_file_path = os.path.join(app.files_folder, pom_info['output_jar'])
            shutil.copyfile(output_jar_path, new_file_path)
            copied_files.append(new_file_path)

        click.echo("Copied %s files to %s [%s]" % (
            len(copied_files), app.files_folder, ",".join(get_filename(name) for name in copied_files)
        ))

        app.copied_files = copied_files


copy_operation_plugin = CopyOperation()
